import json
from datetime import datetime

from django.http import JsonResponse
from django.shortcuts import get_object_or_404, render
from django.utils import timezone
from django.views.decorators.http import require_http_methods

from .models import Game
from .services import HLTBClient, IGDBClient


def turbo_stream_request(request):
    return request.headers.get('Turbo-Frame') or request.headers.get('Turbo-Visit')


def home(request):
    games = Game.objects.filter(added__isnull=False).order_by('-added')
    return render(request, 'playa/home.html', {'games': games})


def game_detail(request, pk):
    game = get_object_or_404(Game, pk=pk)
    return render(
        request,
        'playa/game_detail.html',
        {'game': game, 'status_choices': Game.Status.choices},
    )


def game_search(request):
    if request.method == 'GET':
        query = request.GET.get('q', '')
        if query and len(query) > 2:
            igdb_client = IGDBClient()
            games = igdb_client.search_games(query)

            results = []
            for game in games:
                results.append(
                    {
                        'title': game['name'],
                        'igdb_id': game['id'],
                        'release_date': datetime.fromtimestamp(game.get('first_release_date', 0)),
                        'platforms': [p['name'] for p in game.get('platforms', [])],
                        'poster_url': game.get('cover', {})
                        .get('url', '')
                        .replace('t_thumb', 't_cover_big'),
                        'game_modes': [m['name'] for m in game.get('game_modes', [])],
                    }
                )

            if turbo_stream_request(request):
                return render(
                    request,
                    'playa/game_search.html',
                    {'results': results, 'query': query},
                )

            return render(request, 'playa/game_search.html', {'results': results, 'query': query})

    return render(request, 'playa/game_search.html')


def add_game(request):
    if request.method == 'POST':
        try:
            igdb_id = request.POST.get('igdb_id')

            # Check if game already exists
            if Game.objects.filter(igdb_id=igdb_id).exists():
                return JsonResponse(
                    {'status': 'error', 'message': 'Game already in your collection'}
                )

            igdb_client = IGDBClient()
            game_data = igdb_client.get_game_details(igdb_id)

            # Try HLTB first
            hltb_client = HLTBClient()
            hltb_result = hltb_client.search_game(game_data['name'])

            if hltb_result:
                normally = float(hltb_result.main_extra) if hltb_result.main_extra else None
                hastily = float(hltb_result.main_story) if hltb_result.main_story else None
                completely = float(hltb_result.completionist) if hltb_result.completionist else None
            else:
                # Fall back to IGDB
                time_to_beat = igdb_client.get_time_to_beat(igdb_id)
                normally = time_to_beat.get('normally', 0) / 3600 if time_to_beat else None
                hastily = time_to_beat.get('hastily', 0) / 3600 if time_to_beat else None
                completely = time_to_beat.get('completely', 0) / 3600 if time_to_beat else None

            game = Game(
                title=game_data['name'],
                igdb_id=igdb_id,
                release_date=datetime.fromtimestamp(game_data.get('first_release_date', 0)),
                platforms=', '.join(p['name'] for p in game_data.get('platforms', [])),
                poster_url=game_data.get('cover', {})
                .get('url', '')
                .replace('t_thumb', 't_cover_big'),
                game_modes=', '.join(m['name'] for m in game_data.get('game_modes', [])),
                normally=normally,
                hastily=hastily,
                completely=completely,
            )
            game.save()

            return JsonResponse(
                {
                    'status': 'success',
                    'message': f'Successfully added {game.title} to your collection!',
                }
            )
        except Exception:
            return JsonResponse(
                {
                    'status': 'error',
                    'message': 'An error occurred while adding the game',
                },
                status=500,
            )

    return JsonResponse({'status': 'error', 'message': 'Invalid request method'}, status=400)


@require_http_methods(['POST'])
def update_game(request, pk):
    try:
        data = json.loads(request.body)
        game = get_object_or_404(Game, pk=pk)

        field = data.get('field')
        value = data.get('value')

        if field in ['status', 'owned_platforms', 'progress', 'rating', 'notes']:
            if field in ['progress', 'rating'] and value:
                value = int(value)
            setattr(game, field, value)

            if field != 'notes' and value:  # Update last_played for all fields except notes
                game.last_played = timezone.now()

            game.save()

            return JsonResponse({'status': 'success', 'message': 'Game updated successfully'})

        return JsonResponse({'status': 'error', 'message': 'Invalid field'}, status=400)

    except Exception as e:
        return JsonResponse({'status': 'error', 'message': str(e)}, status=500)
